@@ -2,13 +2,15 @@ package com.microsoft.codepush.react;
 
 import android.app.Activity;
 import android.content.Context;
-import android.content.Intent;
 import android.os.AsyncTask;
+import android.os.Handler;
+import android.os.Looper;
 import android.provider.Settings;
 import android.view.Choreographer;
 
 import com.facebook.react.ReactActivity;
 import com.facebook.react.ReactInstanceManager;
+import com.facebook.react.bridge.Arguments;
 import com.facebook.react.bridge.LifecycleEventListener;
 import com.facebook.react.bridge.Promise;
 import com.facebook.react.bridge.ReactApplicationContext;
@@ -16,7 +18,6 @@ import com.facebook.react.bridge.ReactContextBaseJavaModule;
 import com.facebook.react.bridge.ReactMethod;
 import com.facebook.react.bridge.ReadableMap;
 import com.facebook.react.bridge.WritableMap;
-import com.facebook.react.bridge.WritableNativeMap;
 import com.facebook.react.modules.core.DeviceEventManagerModule;
 import com.facebook.react.uimanager.ReactChoreographer;
 
@@ -33,10 +34,10 @@ import java.util.Map;
 
 public class CodePushNativeModule extends ReactContextBaseJavaModule {
     private String mBinaryContentsHash = null;
-    private String mClientUniqueId = null;    
+    private String mClientUniqueId = null;
     private LifecycleEventListener mLifecycleEventListener = null;
     private int mMinimumBackgroundDuration = 0;
-    
+
     private CodePush mCodePush;
     private SettingsManager mSettingsManager;
     private CodePushTelemetryManager mTelemetryManager;
@@ -78,16 +79,13 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
         return "CodePush";
     }
 
-    private boolean isReactApplication(Context context) {
-        Class<?> reactApplicationClass = tryGetClass(REACT_APPLICATION_CLASS_NAME);
-        if (reactApplicationClass != null && reactApplicationClass.isInstance(context)) {
-            return true;
+    private void loadBundleLegacy() {
+        final Activity currentActivity = getCurrentActivity();
+        if (currentActivity == null) {
+            // The currentActivity can be null if it is backgrounded / destroyed, so we simply
+            // no-op to prevent any null pointer exceptions.
+            return;
         }
-
-        return false;
-    }
-
-    private void loadBundleLegacy(final Activity currentActivity) {
         mCodePush.invalidateCurrentInstance();
 
         currentActivity.runOnUiThread(new Runnable() {
@@ -98,78 +96,123 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
         });
     }
 
+    // Use reflection to find and set the appropriate fields on ReactInstanceManager. See #556 for a proposal for a less brittle way
+    // to approach this.
+    private void setJSBundle(ReactInstanceManager instanceManager, String latestJSBundleFile) throws NoSuchFieldException, IllegalAccessException {
+        try {
+            Field bundleLoaderField = instanceManager.getClass().getDeclaredField("mBundleLoader");
+            Class<?> jsBundleLoaderClass = Class.forName("com.facebook.react.cxxbridge.JSBundleLoader");
+            Method createFileLoaderMethod = null;
+
+            Method[] methods = jsBundleLoaderClass.getDeclaredMethods();
+            for (Method method : methods) {
+                if (method.getName() == "createFileLoader") {
+                    createFileLoaderMethod = method;
+                    break;
+                }
+            }
+
+            if (createFileLoaderMethod == null) {
+                throw new NoSuchMethodException("Could not find a recognized 'createFileLoader' method");
+            }
+
+            int numParameters = createFileLoaderMethod.getGenericParameterTypes().length;
+            Object latestJSBundleLoader;
+
+            if (numParameters == 1) {
+                // RN >= v0.34
+                latestJSBundleLoader = createFileLoaderMethod.invoke(jsBundleLoaderClass, latestJSBundleFile);
+            } else if (numParameters == 2) {
+                // RN >= v0.31 && RN < v0.34
+                latestJSBundleLoader = createFileLoaderMethod.invoke(jsBundleLoaderClass, getReactApplicationContext(), latestJSBundleFile);
+            } else {
+                throw new NoSuchMethodException("Could not find a recognized 'createFileLoader' method");
+            }
+
+            bundleLoaderField.setAccessible(true);
+            bundleLoaderField.set(instanceManager, latestJSBundleLoader);
+        } catch (Exception e) {
+            // RN < v0.31
+            Field jsBundleField = instanceManager.getClass().getDeclaredField("mJSBundleFile");
+            jsBundleField.setAccessible(true);
+            jsBundleField.set(instanceManager, latestJSBundleFile);
+        }
+    }
+
     private void loadBundle() {
         mCodePush.clearDebugCacheIfNeeded();
-        final Activity currentActivity = getCurrentActivity();
-
-        if (currentActivity == null) {
-            // The currentActivity can be null if it is backgrounded / destroyed, so we simply
-            // no-op to prevent any null pointer exceptions.
-            return;
-        } else if (!ReactActivity.class.isInstance(currentActivity)) {
-            // Our preferred reload logic relies on the user's Activity inheriting
-            // from the core ReactActivity class, so if it doesn't, we fallback
-            // early to our legacy behavior.
-            loadBundleLegacy(currentActivity);
-        } else {
-            try {
-                ReactActivity reactActivity = (ReactActivity)currentActivity;
-                ReactInstanceManager instanceManager;
-
-                // #1) Get the ReactInstanceManager instance, which is what includes the
-                //     logic to reload the current React context.
-                try {
-                    // In RN 0.29, the "mReactInstanceManager" field yields a null value, so we try
-                    // to get the instance manager via the ReactNativeHost, which only exists in 0.29.
-                    Method getApplicationMethod = ReactActivity.class.getMethod("getApplication");
-                    Object reactApplication = getApplicationMethod.invoke(reactActivity);
-                    Class<?> reactApplicationClass = tryGetClass(REACT_APPLICATION_CLASS_NAME);
-                    Method getReactNativeHostMethod = reactApplicationClass.getMethod("getReactNativeHost");
-                    Object reactNativeHost = getReactNativeHostMethod.invoke(reactApplication);
-                    Class<?> reactNativeHostClass = tryGetClass(REACT_NATIVE_HOST_CLASS_NAME);
-                    Method getReactInstanceManagerMethod = reactNativeHostClass.getMethod("getReactInstanceManager");
-                    instanceManager = (ReactInstanceManager)getReactInstanceManagerMethod.invoke(reactNativeHost);
-                } catch (Exception e) {
-                    // The React Native version might be older than 0.29, so we try to get the
-                    // instance manager via the "mReactInstanceManager" field.
-                    Field instanceManagerField = ReactActivity.class.getDeclaredField("mReactInstanceManager");
-                    instanceManagerField.setAccessible(true);
-                    instanceManager = (ReactInstanceManager)instanceManagerField.get(reactActivity);
-                }
-
-                String latestJSBundleFile = mCodePush.getJSBundleFileInternal(mCodePush.getAssetsBundleFileName());
-
-                // #2) Update the locally stored JS bundle file path
-                Field jsBundleField = instanceManager.getClass().getDeclaredField("mJSBundleFile");
-                jsBundleField.setAccessible(true);
-                jsBundleField.set(instanceManager, latestJSBundleFile);
-
-                // #3) Get the context creation method and fire it on the UI thread (which RN enforces)
-                final Method recreateMethod = instanceManager.getClass().getMethod("recreateReactContextInBackground");
-
-                final ReactInstanceManager finalizedInstanceManager = instanceManager;
-                reactActivity.runOnUiThread(new Runnable() {
-                    @Override
-                    public void run() {
-                        try {
-                            recreateMethod.invoke(finalizedInstanceManager);
-                            mCodePush.initializeUpdateAfterRestart();
-                        }
-                        catch (Exception e) {
-                            // The recreation method threw an unknown exception
-                            // so just simply fallback to restarting the Activity
-                            loadBundleLegacy(currentActivity);
-                        }
-                    }
-                });
-            } catch (Exception e) {
-                // Our reflection logic failed somewhere
-                // so fall back to restarting the Activity
-                loadBundleLegacy(currentActivity);
+        try {
+            // #1) Get the ReactInstanceManager instance, which is what includes the
+            //     logic to reload the current React context.
+            final ReactInstanceManager instanceManager = resolveInstanceManager();
+            if (instanceManager == null) {
+                return;
             }
+
+            String latestJSBundleFile = mCodePush.getJSBundleFileInternal(mCodePush.getAssetsBundleFileName());
+
+            // #2) Update the locally stored JS bundle file path
+            setJSBundle(instanceManager, latestJSBundleFile);
+
+            // #3) Get the context creation method and fire it on the UI thread (which RN enforces)
+            final Method recreateMethod = instanceManager.getClass().getMethod("recreateReactContextInBackground");
+            new Handler(Looper.getMainLooper()).post(new Runnable() {
+                @Override
+                public void run() {
+                    try {
+                        recreateMethod.invoke(instanceManager);
+                        mCodePush.initializeUpdateAfterRestart();
+                    } catch (Exception e) {
+                        // The recreation method threw an unknown exception
+                        // so just simply fallback to restarting the Activity (if it exists)
+                        loadBundleLegacy();
+                    }
+                }
+            });
+
+        } catch (Exception e) {
+            // Our reflection logic failed somewhere
+            // so fall back to restarting the Activity (if it exists)
+            loadBundleLegacy();
         }
     }
 
+    // Use reflection to find the ReactInstanceManager. See #556 for a proposal for a less brittle way to approach this.
+    private ReactInstanceManager resolveInstanceManager() throws NoSuchFieldException, IllegalAccessException {
+        ReactInstanceManager instanceManager = CodePush.getReactInstanceManager();
+        if (instanceManager != null) {
+            return instanceManager;
+        }
+
+        final Activity currentActivity = getCurrentActivity();
+        if (currentActivity == null) {
+            return null;
+        }
+        try {
+            // In RN >=0.29, the "mReactInstanceManager" field yields a null value, so we try
+            // to get the instance manager via the ReactNativeHost, which only exists in 0.29.
+            Method getApplicationMethod = ReactActivity.class.getMethod("getApplication");
+            Object reactApplication = getApplicationMethod.invoke(currentActivity);
+            Class<?> reactApplicationClass = tryGetClass(REACT_APPLICATION_CLASS_NAME);
+            Method getReactNativeHostMethod = reactApplicationClass.getMethod("getReactNativeHost");
+            Object reactNativeHost = getReactNativeHostMethod.invoke(reactApplication);
+            Class<?> reactNativeHostClass = tryGetClass(REACT_NATIVE_HOST_CLASS_NAME);
+            Method getReactInstanceManagerMethod = reactNativeHostClass.getMethod("getReactInstanceManager");
+            instanceManager = (ReactInstanceManager)getReactInstanceManagerMethod.invoke(reactNativeHost);
+        } catch (Exception e) {
+            // The React Native version might be older than 0.29, or the activity does not
+            // extend ReactActivity, so we try to get the instance manager via the
+            // "mReactInstanceManager" field.
+            Class instanceManagerHolderClass = currentActivity instanceof ReactActivity
+                    ? ReactActivity.class
+                    : currentActivity.getClass();
+            Field instanceManagerField = instanceManagerHolderClass.getDeclaredField("mReactInstanceManager");
+            instanceManagerField.setAccessible(true);
+            instanceManager = (ReactInstanceManager)instanceManagerField.get(currentActivity);
+        }
+        return instanceManager;
+    }
+
     private Class tryGetClass(String className) {
         try {
             return Class.forName(className);
@@ -184,8 +227,8 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
             @Override
             protected Void doInBackground(Void... params) {
                 try {
-                    WritableMap mutableUpdatePackage = CodePushUtils.convertReadableMapToWritableMap(updatePackage);
-                    mutableUpdatePackage.putString(CodePushConstants.BINARY_MODIFIED_TIME_KEY, "" + mCodePush.getBinaryResourcesModifiedTime());
+                    JSONObject mutableUpdatePackage = CodePushUtils.convertReadableToJsonObject(updatePackage);
+                    CodePushUtils.setJSONValueForKey(mutableUpdatePackage, CodePushConstants.BINARY_MODIFIED_TIME_KEY, "" + mCodePush.getBinaryResourcesModifiedTime());
                     mUpdateManager.downloadPackage(mutableUpdatePackage, mCodePush.getAssetsBundleFileName(), new DownloadProgressCallback() {
                         private boolean hasScheduledNextFrame = false;
                         private DownloadProgress latestDownloadProgress = null;
@@ -232,14 +275,14 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
                         }
                     });
 
-                    WritableMap newPackage = mUpdateManager.getPackage(CodePushUtils.tryGetString(updatePackage, CodePushConstants.PACKAGE_HASH_KEY));
-                    promise.resolve(newPackage);
+                    JSONObject newPackage = mUpdateManager.getPackage(CodePushUtils.tryGetString(updatePackage, CodePushConstants.PACKAGE_HASH_KEY));
+                    promise.resolve(CodePushUtils.convertJsonObjectToWritable(newPackage));
                 } catch (IOException e) {
                     e.printStackTrace();
                     promise.reject(e);
                 } catch (CodePushInvalidUpdateException e) {
                     e.printStackTrace();
-                    mSettingsManager.saveFailedUpdate(updatePackage);
+                    mSettingsManager.saveFailedUpdate(CodePushUtils.convertReadableToJsonObject(updatePackage));
                     promise.reject(e);
                 }
 
@@ -252,7 +295,7 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
 
     @ReactMethod
     public void getConfiguration(Promise promise) {
-        WritableNativeMap configMap = new WritableNativeMap();
+        WritableMap configMap =  Arguments.createMap();
         configMap.putString("appVersion", mCodePush.getAppVersion());
         configMap.putString("clientUniqueId", mClientUniqueId);
         configMap.putString("deploymentKey", mCodePush.getDeploymentKey());
@@ -271,7 +314,7 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
         AsyncTask<Void, Void, Void> asyncTask = new AsyncTask<Void, Void, Void>() {
             @Override
             protected Void doInBackground(Void... params) {
-                WritableMap currentPackage = mUpdateManager.getCurrentPackage();
+                JSONObject currentPackage = mUpdateManager.getCurrentPackage();
 
                 if (currentPackage == null) {
                     promise.resolve("");
@@ -280,8 +323,8 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
 
                 Boolean currentUpdateIsPending = false;
 
-                if (currentPackage.hasKey(CodePushConstants.PACKAGE_HASH_KEY)) {
-                    String currentHash = currentPackage.getString(CodePushConstants.PACKAGE_HASH_KEY);
+                if (currentPackage.has(CodePushConstants.PACKAGE_HASH_KEY)) {
+                    String currentHash = currentPackage.optString(CodePushConstants.PACKAGE_HASH_KEY, null);
                     currentUpdateIsPending = mSettingsManager.isPendingUpdate(currentHash);
                 }
 
@@ -292,7 +335,14 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
                 } else if (updateState == CodePushUpdateState.RUNNING.getValue() && currentUpdateIsPending) {
                     // The caller wants the running update, but the current
                     // one is pending, so we need to grab the previous.
-                    promise.resolve(mUpdateManager.getPreviousPackage());
+                    JSONObject previousPackage = mUpdateManager.getPreviousPackage();
+
+                    if (previousPackage == null) {
+                        promise.resolve("");
+                        return null;
+                    }
+
+                    promise.resolve(CodePushUtils.convertJsonObjectToWritable(previousPackage));
                 } else {
                     // The current package satisfies the request:
                     // 1) Caller wanted a pending, and there is a pending update
@@ -302,12 +352,12 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
                         // This only matters in Debug builds. Since we do not clear "outdated" updates,
                         // we need to indicate to the JS side that somehow we have a current update on
                         // disk that is not actually running.
-                        currentPackage.putBoolean("_isDebugOnly", true);
+                        CodePushUtils.setJSONValueForKey(currentPackage, "_isDebugOnly", true);
                     }
 
                     // Enable differentiating pending vs. non-pending updates
-                    currentPackage.putBoolean("isPending", currentUpdateIsPending);
-                    promise.resolve(currentPackage);
+                    CodePushUtils.setJSONValueForKey(currentPackage, "isPending", currentUpdateIsPending);
+                    promise.resolve(CodePushUtils.convertJsonObjectToWritable(currentPackage));
                 }
 
                 return null;
@@ -339,9 +389,9 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
                         }
                     }
                 } else if (mCodePush.didUpdate()) {
-                    WritableMap currentPackage = mUpdateManager.getCurrentPackage();
+                    JSONObject currentPackage = mUpdateManager.getCurrentPackage();
                     if (currentPackage != null) {
-                        WritableMap newPackageStatusReport = mTelemetryManager.getUpdateReport(currentPackage);
+                        WritableMap newPackageStatusReport = mTelemetryManager.getUpdateReport(CodePushUtils.convertJsonObjectToWritable(currentPackage));
                         if (newPackageStatusReport != null) {
                             promise.resolve(newPackageStatusReport);
                             return null;
@@ -374,7 +424,7 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
         AsyncTask<Void, Void, Void> asyncTask = new AsyncTask<Void, Void, Void>() {
             @Override
             protected Void doInBackground(Void... params) {
-                mUpdateManager.installPackage(updatePackage, mSettingsManager.isPendingUpdate(null));
+                mUpdateManager.installPackage(CodePushUtils.convertReadableToJsonObject(updatePackage), mSettingsManager.isPendingUpdate(null));
 
                 String pendingHash = CodePushUtils.tryGetString(updatePackage, CodePushConstants.PACKAGE_HASH_KEY);
                 if (pendingHash == null) {
@@ -497,4 +547,4 @@ public class CodePushNativeModule extends ReactContextBaseJavaModule {
             }
         }
     }
-}
\ No newline at end of file
+}
