@@ -10,9 +10,9 @@ namespace CodePush.ReactNative
 {
     internal class CodePushUtils
     {
-        internal async static Task<JObject> GetJObjectFromFile(StorageFile file)
+        internal async static Task<JObject> GetJObjectFromFileAsync(StorageFile file)
         {
-            string jsonString = await FileIO.ReadTextAsync(file);
+            string jsonString = await FileIO.ReadTextAsync(file).AsTask().ConfigureAwait(false);
             if (jsonString.Length == 0)
             {
                 return new JObject();
